@@ -1,34 +1,34 @@
 import json
 import logging
 import pickletools
+import threading
 import time
 import traceback
 
-from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
-from zope.app.publication.zopepublication import ZopePublication, Cleanup
-from zope.publisher.browser import BrowserView
-from zope.publisher.interfaces.browser import IBrowserRequest
-from zope.component import adapter, queryUtility
-from zope.interface import Interface
-from zope.security.proxy import removeSecurityProxy
-from zope.cachedescriptors.property import Lazy
-from zope.exceptions.interfaces import UserError
-from ZODB.utils import p64, u64, tid_repr, oid_repr
+import transaction
+from persistent import Persistent
+from persistent.TimeStamp import TimeStamp
 from ZODB.Connection import Connection
 from ZODB.interfaces import IDatabase
 from ZODB.POSException import POSKeyError
-from persistent import Persistent
-from persistent.TimeStamp import TimeStamp
-import transaction
+from ZODB.utils import oid_repr, p64, tid_repr, u64
+from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
+from zope.app.publication.zopepublication import Cleanup, ZopePublication
+from zope.cachedescriptors.property import Lazy
+from zope.component import adapter, queryUtility
+from zope.exceptions.interfaces import UserError
+from zope.interface import Interface
+from zope.publisher.browser import BrowserView
+from zope.publisher.interfaces.browser import IBrowserRequest
+from zope.security.proxy import removeSecurityProxy
 
-from zodbbrowser import __version__, __homepage__
-from zodbbrowser.compat import escape, StringIO, BytesIO
+from zodbbrowser import __homepage__, __version__
+from zodbbrowser.compat import BytesIO, StringIO, escape
 from zodbbrowser.diff import compareDictsHTML
-from zodbbrowser.history import ZodbObjectHistory
-from zodbbrowser.interfaces import IDatabaseHistory
-from zodbbrowser.interfaces import IValueRenderer
+from zodbbrowser.history import getObjectHistory
+from zodbbrowser.interfaces import IDatabaseHistory, IValueRenderer
 from zodbbrowser.state import ZodbObjectState
-from zodbbrowser.value import pruneTruncations, TRUNCATIONS
+from zodbbrowser.value import TRUNCATIONS, pruneTruncations
 
 
 log = logging.getLogger("zodbbrowser")
@@ -118,8 +118,35 @@ class VeryCarefulView(BrowserView):
                 transaction.abort()
 
 
+class TimedMixin:
+
+    _started = _last_mark = None
+
+    def reset_mark(self, message=None):
+        self._started = self._last_mark = time.time()
+        if message:
+            log.debug('%s %-14s %s' % (self.thread_tag(), '-' * 14, message))
+
+    def time_elapsed(self):
+        return time.time() - self._started
+
+    def thread_tag(self):
+        return '[%s]' % threading.current_thread().name
+
+    def debug_mark(self, mark):
+        now = time.time()
+        if self._last_mark is None:
+            self._last_mark, interval = now, 0
+        else:
+            self._last_mark, interval = now, now - self._last_mark
+        log.debug('%s %-14s %s' % (self.thread_tag(), formatTime(interval), mark))
+
+    def renderingTime(self):
+        return formatTime(self.time_elapsed()) + ' |'
+
+
 @adapter(Interface, IBrowserRequest)
-class ZodbInfoView(VeryCarefulView):
+class ZodbInfoView(TimedMixin, VeryCarefulView):
     """Zodb browser view"""
 
     template = ViewPageTemplateFile('templates/zodbinfo.pt')
@@ -129,12 +156,14 @@ class ZodbInfoView(VeryCarefulView):
     homepage = __homepage__
 
     def render(self):
-        self._started = time.time()
+        self.reset_mark(getFullRequestUrl(self.request))
         pruneTruncations()
         self.obj = self.selectObjectToView()
-        # Not using IObjectHistory(self.obj) because LP#1185175
-        self.history = ZodbObjectHistory(self.obj)
+        self.debug_mark('- loading object history')
+        # Not using IObjectHistory(self.obj) because LP: #1185175
+        self.history = getObjectHistory(self.obj)
         self.latest = True
+        self.debug_mark('- loading object state')
         if self.request.get('tid'):
             self.state = ZodbObjectState(self.obj,
                                          p64(int(self.request['tid'], 0)),
@@ -160,10 +189,11 @@ class ZodbInfoView(VeryCarefulView):
             # will show confirmation prompt
             return self.confirmation_template()
 
-        return self.template()
-
-    def renderingTime(self):
-        return '%.3fs |' % (time.time() - self._started)
+        self.debug_mark('- rendering')
+        try:
+            return self.template()
+        finally:
+            self.debug_mark('- done (%s)' % formatTime(self.time_elapsed()))
 
     def _redirectToSelf(self):
         self.request.response.redirect(self.getUrl())
@@ -309,6 +339,7 @@ class ZodbInfoView(VeryCarefulView):
         return url
 
     def getBreadcrumbs(self):
+        self.debug_mark('- rendering breadcrumbs')
         breadcrumbs = []
         state = self.state
         seen_root = False
@@ -350,6 +381,7 @@ class ZodbInfoView(VeryCarefulView):
         return ''.join(html)
 
     def getDisassembledPickleData(self):
+        self.debug_mark('- rendering pickle')
         pickle = BytesIO(self.state.pickledState)
         out = StringIO()
         memo = {}
@@ -367,6 +399,7 @@ class ZodbInfoView(VeryCarefulView):
         return out.getvalue()
 
     def listAttributes(self):
+        self.debug_mark('- rendering attributes')
         attrs = self.state.listAttributes()
         if attrs is None:
             return None
@@ -374,6 +407,7 @@ class ZodbInfoView(VeryCarefulView):
                 for name, value in sorted(attrs)]
 
     def listItems(self):
+        self.debug_mark('- rendering items')
         items = self.state.listItems()
         if items is None:
             return None
@@ -397,9 +431,14 @@ class ZodbInfoView(VeryCarefulView):
 
     def listHistory(self):
         """List transactions that modified a persistent object."""
+        if 'nohist' in self.request:
+            return []
+
+        self.debug_mark('- rendering history')
         state = self._loadHistoricalState()
         results = []
         for n, d in enumerate(self.history):
+            self.debug_mark('- rendering history #%s' % n)
             utc_timestamp = str(time.strftime('%Y-%m-%d %H:%M:%S',
                                               time.gmtime(d['time'])))
             local_timestamp = str(time.strftime('%Y-%m-%d %H:%M:%S',
@@ -410,8 +449,8 @@ class ZodbInfoView(VeryCarefulView):
                 user_location = None
                 user_id = d['user_name']
             url = self.getUrl(tid=u64(d['tid']))
-            current = (d['tid'] == self.state.tid and
-                       self.state.requestedTid is not None)
+            current = (d['tid'] == self.state.tid
+                       and self.state.requestedTid is not None)
             curState = state[n]['state']
             oldState = state[n + 1]['state']
             diff = compareDictsHTML(curState, oldState, d['tid'])
@@ -437,7 +476,7 @@ class ZodbInfoView(VeryCarefulView):
 
 
 @adapter(Interface, IBrowserRequest)
-class ZodbHistoryView(VeryCarefulView):
+class ZodbHistoryView(TimedMixin, VeryCarefulView):
     """Zodb history view"""
 
     template = ViewPageTemplateFile('templates/zodbhistory.pt')
@@ -448,16 +487,17 @@ class ZodbHistoryView(VeryCarefulView):
     page_size = 5
 
     def update(self):
-        self._started = time.time()
         pruneTruncations()
         if 'page_size' in self.request:
             self.page_size = max(1, int(self.request['page_size']))
+        self.debug_mark('Loading history')
         self.history = IDatabaseHistory(self.jar)
         self.request.hold(Cleanup(self.history.cleanup))
         if 'page' in self.request:
             self.page = int(self.request['page'])
         elif 'tid' in self.request:
             tid = int(self.request['tid'], 0)
+            self.debug_mark('- finding transaction page')
             self.page = self.findPage(p64(tid))
         else:
             self.page = 0
@@ -468,11 +508,12 @@ class ZodbHistoryView(VeryCarefulView):
         self.first_idx = max(0, self.last_idx - self.page_size)
 
     def render(self):
+        self.reset_mark(getFullRequestUrl(self.request))
         self.update()
-        return self.template()
-
-    def renderingTime(self):
-        return '%.3fs |' % (time.time() - self._started)
+        try:
+            return self.template()
+        finally:
+            self.debug_mark('- done (%s)' % formatTime(self.time_elapsed()))
 
     def getUrl(self, tid=None):
         url = "@@zodbbrowser_history"
@@ -491,6 +532,7 @@ class ZodbHistoryView(VeryCarefulView):
             return (len(self.history) - pos - 1) // self.page_size
 
     def listHistory(self):
+        self.debug_mark('- listing history')
         if 'tid' in self.request:
             requested_tid = p64(int(self.request['tid'], 0))
         else:
@@ -518,26 +560,40 @@ class ZodbHistoryView(VeryCarefulView):
                 size = d._tend - d._tpos
             except AttributeError:
                 size = None
+            self.debug_mark('- listing 0x%x (%s)' % (utid, formatSize(size)))
             ext = d.extension if isinstance(d.extension, dict) else {}
             objects = []
-            for record in d:
-                obj = self.jar.get(record.oid)
+            for idx, record in enumerate(d):
                 url = "@@zodbbrowser?oid=0x%x&tid=0x%x" % (u64(record.oid),
                                                            utid)
-                objects.append(dict(
-                    oid=u64(record.oid),
-                    path=getObjectPath(obj, d.tid),
-                    oid_repr=oid_repr(record.oid),
-                    class_repr=getObjectType(obj),
-                    url=url,
-                    repr=IValueRenderer(obj).render(d.tid),
-                ))
+                if 'fast' in self.request or (
+                        self.time_elapsed() > 10
+                        and idx > 10
+                        and 'full' not in self.request):
+                    objects.append(dict(
+                        oid=u64(record.oid),
+                        path='0x%x' % u64(record.oid),
+                        oid_repr=oid_repr(record.oid),
+                        class_repr='',
+                        url=url,
+                        repr='(view object)',
+                    ))
+                else:
+                    obj = self.jar.get(record.oid)
+                    objects.append(dict(
+                        oid=u64(record.oid),
+                        path=getObjectPath(obj, d.tid),
+                        oid_repr=oid_repr(record.oid),
+                        class_repr=getObjectType(obj),
+                        url=url,
+                        repr=IValueRenderer(obj).render(d.tid),
+                    ))
             if len(objects) == 1:
                 summary = '1 object record'
             else:
                 summary = '%d object records' % len(objects)
             if size is not None:
-                summary += ' (%d bytes)' % size
+                summary += ' (%s)' % formatSize(size)
             results.append(dict(
                 index=(self.first_idx + n + 1),
                 utc_timestamp=utc_timestamp,
@@ -556,6 +612,7 @@ class ZodbHistoryView(VeryCarefulView):
             ))
         if results and not requested_tid and self.page == 0:
             results[-1]['current'] = True
+        self.debug_mark('- back to rendering')
         return results[::-1]
 
 
@@ -601,3 +658,30 @@ def getObjectPath(obj, tid):
             break
     return ''.join(path[::-1])
 
+
+def formatTime(seconds):
+    min, sec = divmod(seconds, 60)
+    if min > 0:
+        return '%dm %.3fs' % (int(min), sec)
+    else:
+        return '%.3fs' % seconds
+
+
+def formatSize(size):
+    if size is None:
+        return 'n/a bytes'
+    format_string = '%.0f %s'
+    for units in 'bytes', 'KiB', 'MiB', 'GiB':
+        if size < 1024:
+            break
+        size = size / 1024.
+        format_string = '%.1f %s'
+    return format_string % (size, units)
+
+
+def getFullRequestUrl(request):
+    qs = request.environment.get('QUERY_STRING', '')
+    if qs:
+        return '%s?%s' % (request.URL, qs)
+    else:
+        return str(request.URL)
