@@ -14,21 +14,22 @@ import unittest
 import transaction
 from lxml.html import fromstring, tostring
 from persistent import Persistent
-from ZODB.POSException import ReadOnlyError
-from ZODB.FileStorage.FileStorage import FileStorage
 from ZODB.DB import DB
-from zope.testing.renormalizing import RENormalizing
-from zope.testbrowser.browser import Browser as _Browser, HTTPError
-from zope.testbrowser.interfaces import IBrowser
-from zope.app.testing import setup
-from zope.app.publication.zopepublication import ZopePublication
-from zope.app.folder.folder import Folder
-from zope.app.appsetup.interfaces import DatabaseOpened
+from ZODB.FileStorage.FileStorage import FileStorage
+from ZODB.POSException import ReadOnlyError
 from zope.app.appsetup.bootstrap import bootStrapSubscriber
+from zope.app.appsetup.interfaces import DatabaseOpened
+from zope.app.folder.folder import Folder
+from zope.app.publication.zopepublication import ZopePublication
+from zope.app.testing import setup
 from zope.interface import Interface, implementer_only
+from zope.testbrowser.browser import Browser as _Browser
+from zope.testbrowser.browser import HTTPError
+from zope.testbrowser.interfaces import IBrowser
+from zope.testing.renormalizing import RENormalizing
 
 from zodbbrowser import standalone
-from zodbbrowser.compat import basestring, StringIO, escape
+from zodbbrowser.compat import StringIO, basestring, escape
 from zodbbrowser.standalone import main, serve_forever, stop_serving
 from zodbbrowser.value import resetTruncations
 
@@ -388,6 +389,7 @@ def test_suite():
         (re.compile(r'\btid=[0-9xA-Fa-f]+'), 'tid=XXXXXXXXXXXXXXXXXX'),
         (re.compile(r'\boid=[0-9xA-Fa-f]+'), 'oid=XX'),
         (re.compile(r'\boid [0-9xA-Fa-f]+'), 'oid XX'),
+        (re.compile(r'\b0x[0-9xA-Fa-f]+$', re.MULTILINE), '0xXX'),
         (re.compile(r'\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d[.]\d\d\d\d\d\d'),
             'YYYY-MM-DD HH:MM:SS.SSSSSS'),
         (re.compile(r'\d\d\d\d-\d\d-\d\d \d\d:\d\d:\d\d'),
@@ -402,6 +404,10 @@ def test_suite():
         # Python 3 has unicode strings without u prefixes
         (re.compile(r"u('.*')"), r'\1'),
         (re.compile(r": u'$", re.MULTILINE), ": '"),
+        # Python 2 says <type '...'>, Python 3 says '<class ...>'
+        (re.compile(r"&lt;type 'BTrees\.OOBTree\.OOBTree'&gt;"),
+         "&lt;class 'BTrees.OOBTree.OOBTree'&gt;"),
+
     ])
     optionflags = (doctest.REPORT_ONLY_FIRST_FAILURE |
                    doctest.REPORT_NDIFF | doctest.NORMALIZE_WHITESPACE)
