@@ -1,27 +1,39 @@
-import unittest
-import transaction
 import gc
 import json
+import unittest
 
-from ZODB.interfaces import IDatabase
-from ZODB.utils import u64, p64, tid_repr, oid_repr
+import transaction
 from persistent import Persistent
+from ZODB.interfaces import IDatabase
+from ZODB.utils import oid_repr, p64, tid_repr, u64
 from zope.app.container.btree import BTreeContainer
 from zope.app.container.interfaces import IContained
 from zope.app.testing import setup
-from zope.component import provideAdapter, getGlobalSiteManager
+from zope.component import getGlobalSiteManager, provideAdapter
 from zope.exceptions.interfaces import UserError
 from zope.interface import implementer
 from zope.publisher.browser import TestRequest
-from zope.traversing.interfaces import IContainmentRoot
-from zope.security.proxy import Proxy
 from zope.security.checker import ProxyFactory
+from zope.security.proxy import Proxy
+from zope.traversing.interfaces import IContainmentRoot
 
 from zodbbrowser.browser import (
-    ZodbObjectAttribute, VeryCarefulView, ZodbInfoView, ZodbHistoryView,
-    getObjectType, getObjectTypeShort, getObjectPath)
+    VeryCarefulView,
+    ZodbHistoryView,
+    ZodbInfoView,
+    ZodbObjectAttribute,
+    formatSize,
+    formatTime,
+    getObjectPath,
+    getObjectType,
+    getObjectTypeShort,
+)
 from zodbbrowser.btreesupport import EmptyOOBTreeState
-from zodbbrowser.history import ZodbObjectHistory, ZodbHistory, getIterableStorage
+from zodbbrowser.history import (
+    ZodbHistory,
+    ZodbObjectHistory,
+    getIterableStorage,
+)
 from zodbbrowser.state import GenericState, ZodbObjectState
 from zodbbrowser.testing import SimpleValueRenderer
 
@@ -555,6 +567,7 @@ class TestZodbHistoryView(RealDatabaseTest):
         request = TestRequest(**kw)
         view = ZodbHistoryView(self.root, request)
         view.template = lambda: ''
+        view.time_elapsed = lambda: 0
         return view
 
     def test_render(self):
@@ -644,6 +657,19 @@ class TestHelperFunctions(unittest.TestCase):
         self.assertEqual(getObjectTypeShort(ProxyFactory(NonpersistentStub())),
                          Proxy.__name__ + ' - NonpersistentStub')
 
+    def test_formatTime(self):
+        self.assertEqual(formatTime(0), '0.000s')
+        self.assertEqual(formatTime(1.5), '1.500s')
+        self.assertEqual(formatTime(60.25), '1m 0.250s')
+
+    def test_formatSize(self):
+        self.assertEqual(formatSize(None), 'n/a bytes')
+        self.assertEqual(formatSize(0), '0 bytes')
+        self.assertEqual(formatSize(15), '15 bytes')
+        self.assertEqual(formatSize(1024), '1.0 KiB')
+        self.assertEqual(formatSize(1024**2), '1.0 MiB')
+        self.assertEqual(formatSize(1024**3), '1.0 GiB')
+
 
 class TestHelperFunctionsWithRealDb(RealDatabaseTest):
 
